# Load libraries
library(tidyverse)
library(data.table)
library(stringr)
library(pbmcapply)
library(sandwich)
library(lmtest)
library(broom)
library(estimatr)
library(speedglm)

# Set working directory
setwd("~/Google Drive/Research/Projects/Current Research/DISTFAIR")

# Set years that we want
years <- 2006:2020

# Set variables to keep
keeps <- c("SENTTOT","SENTYR","NEWRACE","CIRCDIST","CHAP2","COADJLEV","XFOLSOR","TOTCHPTS","GLMIN","XMINSOR","BOOKERCD","SENTRNGE","OFFGUIDE","OFFTYPE2","OFFTYPSB","NEWCNVTN","AGE","MONSEX","NEWCIT","NEWEDUC","SENTIMP","XCRHISSR")

# # Set up start of file path
# files <- paste0("~/Too Big/JUSTFAIR/opafy",str_sub(as.character(years),-2,-1),"nid.csv")
# 
# # Load raw files and combine
# data <- bind_rows(pbmclapply(files,function(x) fread(x, select=keeps)))
# 
# # Save
# write.csv(data, file = "data.csv", row.names = FALSE)

# Load data
data1 <- fread("data.csv")

# Keep SENTYR 2006 at later
# Create boolean for immigration cases
data2 <- data1 %>%
  filter(SENTYR >= 2006) %>%
  mutate(IMMIGRATION = FALSE) %>%
  mutate(IMMIGRATION = replace(IMMIGRATION, is.na(OFFGUIDE) & is.na(OFFTYPE2) & is.na(OFFTYPSB), NA)) %>%
  mutate(IMMIGRATION = replace(IMMIGRATION, (OFFGUIDE == 17) | (OFFTYPE2 == 27) | (OFFTYPSB == 27), TRUE))
  
# Drop cases involving immigration and/or non-US citizens and/or missing citizenship data
data3 <- data2 %>%
  filter(IMMIGRATION == FALSE & NEWCIT == 0) %>%
  dplyr::select(-IMMIGRATION,-NEWCIT,-OFFGUIDE,-OFFTYPE2,-OFFTYPSB)

# Impute 0 for SENTTOT when we know that no prison time was give
# Drop records with still-missing value of SENTTOT
data4 <- data3 %>%
  mutate(SENTTOT = replace(SENTTOT, is.na(SENTTOT) & SENTIMP %in% c(0,3,4), 0)) %>%
  filter(!is.na(SENTTOT)) %>%
  dplyr::select(-SENTIMP)

# Drop cases with SENTTOT > 470
data5 <- data4 %>%
  filter(SENTTOT <= 470)

# Add departures variables
data5 <- data5 %>%
  mutate(K11 = FALSE) %>%
  mutate(K11 = replace(K11, is.na(BOOKERCD) & is.na(SENTRNGE), NA)) %>%
  mutate(K11 = replace(K11, BOOKERCD == 5, TRUE)) %>%
  mutate(K11 = replace(K11, SENTRNGE == 2, TRUE)) %>%
  mutate(K31 = FALSE) %>%
  mutate(K31 = replace(K31, is.na(BOOKERCD) & is.na(SENTRNGE), NA)) %>%
  mutate(K31 = replace(K31, BOOKERCD == 6, TRUE)) %>%
  mutate(K31 = replace(K31, SENTRNGE == 3, TRUE)) %>%
  mutate(GOVDEPART = K11 | K31) %>%
  dplyr::select(-BOOKERCD,-SENTRNGE,-K11,-K31)

# Retain only complete cases
data6 <- data5 %>%
  na.omit

# Cast variables as appropriate (except for CIRCDIST, handled separately)
data7 <- data6 %>%
  mutate(SENTYR = as.factor(SENTYR)) %>%
  mutate(NEWRACE = as.factor(NEWRACE)) %>%
  mutate(CHAP2 = as.factor(CHAP2)) %>%
  mutate(COADJLEV = as.factor(COADJLEV)) %>%
  mutate(XFOLSOR = as.factor(XFOLSOR)) %>%
  mutate(TOTCHPTS = as.factor(TOTCHPTS)) %>%
  mutate(XCRHISSR = as.factor(XCRHISSR)) %>%
  mutate(NEWCNVTN = as.factor(NEWCNVTN)) %>%
  mutate(AGE = as.numeric(AGE)) %>%
  mutate(MONSEX = as.factor(MONSEX)) %>%
  mutate(NEWEDUC = as.factor(NEWEDUC))

# Recode CIRCDIST and NEWRACE
# Get rid of any unused levels
distkey <- read.csv("circdist.csv")
data8 <- data7 %>%
  merge(distkey, by.x = "CIRCDIST", by.y = "circdist") %>%
  dplyr::select(-"CIRCDIST") %>%
  rename("CIRCDIST" = "name") %>%
  mutate(CIRCDIST = factor(CIRCDIST)) %>%
  mutate(NEWRACE = recode_factor(NEWRACE, "1" = "white", "2" = "black", "3" = "hispanic", "6" = "other")) %>%
  droplevels()

# Drop N Mariana Islands due to not enough data (98 records)
data9 <- data8 %>%
  filter(CIRCDIST != "N Mariana Islands")

# Create derived variables
data10 <- data9 %>%
  mutate(MANDMIN = GLMIN > XMINSOR) %>%
  dplyr::select(-GLMIN,-XMINSOR) %>%
  mutate(GRID = paste0("XFOLSOR",XFOLSOR,"XCRHISSR",XCRHISSR)) %>%
  mutate(GRID = factor(GRID)) %>%
  mutate(DISTRACE = paste0("CIRCDIST",CIRCDIST,":NEWRACE",NEWRACE)) %>%
  mutate(DISTRACE = factor(DISTRACE)) %>%
  relocate(SENTTOT,AGE,MONSEX,NEWEDUC,SENTYR,NEWCNVTN,TOTCHPTS,
           CHAP2,COADJLEV,XFOLSOR,GRID,MANDMIN,GOVDEPART,CIRCDIST,NEWRACE,DISTRACE)

# Drop all unused levels
data11 <- data10 %>%
  droplevels

# Choose data for the lm
lmdata <- data11

# Clear out unneeded stuff
remove(list=setdiff(ls(),"lmdata"))

# Load library for speedy lm
#library(speedglm)

# Base
M0 <-  speedlm(SENTTOT ~ NEWRACE, data = lmdata, model = TRUE, fitted = TRUE, y = TRUE)
save(M0, file = "M0.Rdata")
remove(M0)
gc()

# personal demographics + race
M1 <-  speedlm(SENTTOT ~ AGE + MONSEX + NEWEDUC + NEWRACE, data = lmdata, model = TRUE, fitted = TRUE, y = TRUE)
save(M1, file = "M1.Rdata")
remove(M1)
gc()

# personal demographics + year + race
M2 <-  speedlm(SENTTOT ~ AGE + MONSEX + NEWEDUC + SENTYR + NEWRACE, data = lmdata, model = TRUE, fitted = TRUE, y = TRUE)
save(M2, file = "M2.Rdata")
remove(M2)
gc()

# personal demographics + year + plea + race
M3 <-  speedlm(SENTTOT ~ AGE + MONSEX + NEWEDUC + SENTYR + NEWCNVTN + NEWRACE, data = lmdata, model = TRUE, fitted = TRUE, y = TRUE)
save(M3, file = "M3.Rdata")
remove(M3)
gc()

# personal demographics + year + plea + criminal history + race
M4 <-  speedlm(SENTTOT ~ AGE + MONSEX + NEWEDUC + SENTYR + NEWCNVTN + XCRHISSR + NEWRACE, data = lmdata, model = TRUE, fitted = TRUE, y = TRUE)
save(M4, file = "M4.Rdata")
remove(M4)
gc()

# personal demographics + year + plea + criminal history + chap2 + race
M5 <-  speedlm(SENTTOT ~ AGE + MONSEX + NEWEDUC + SENTYR + NEWCNVTN + XCRHISSR + CHAP2 + NEWRACE, data = lmdata, model = TRUE, fitted = TRUE, y = TRUE)
save(M5, file = "M5.Rdata")
remove(M5)
gc()

# personal demographics + year + plea + criminal history + chap3 + race
M6 <-  speedlm(SENTTOT ~ AGE + MONSEX + NEWEDUC + SENTYR + NEWCNVTN + XCRHISSR + COADJLEV + NEWRACE, data = lmdata, model = TRUE, fitted = TRUE, y = TRUE)
save(M6, file = "M6.Rdata")
remove(M6)
gc()

# personal demographics + year + plea + criminal history + final offense level + race
M7 <-  speedlm(SENTTOT ~ AGE + MONSEX + NEWEDUC + SENTYR + NEWCNVTN + XCRHISSR + XFOLSOR + NEWRACE, data = lmdata, model = TRUE, fitted = TRUE, y = TRUE)
save(M7, file = "M7.Rdata")
remove(M7)
gc()

# personal demographics + year + plea + grid + race
M8 <-  speedlm(SENTTOT ~ AGE + MONSEX + NEWEDUC + SENTYR + NEWCNVTN + GRID + NEWRACE, data = lmdata, model = TRUE, fitted = TRUE, y = TRUE)
save(M8, file = "M8.Rdata")
remove(M8)
gc()

# personal demographics + year + plea + grid + mandmin + race
M9 <-  speedlm(SENTTOT ~ AGE + MONSEX + NEWEDUC + SENTYR + NEWCNVTN + GRID + MANDMIN + NEWRACE, data = lmdata, model = TRUE, fitted = TRUE, y = TRUE)
save(M9, file = "M9.Rdata")
remove(M9)
gc()

# personal demographics + year + plea + grid + mandmin + govt departures + race
M10 <- speedlm(SENTTOT ~ AGE + MONSEX + NEWEDUC + SENTYR + NEWCNVTN + GRID + MANDMIN + GOVDEPART + NEWRACE, data = lmdata, model = TRUE, fitted = TRUE, y = TRUE)
save(M10, file = "M10.Rdata")
remove(M10)
gc()

# personal demographics + year + plea + grid + mandmin + govt departures + district + race
M11 <- speedlm(SENTTOT ~ AGE + MONSEX + NEWEDUC + SENTYR + NEWCNVTN + GRID + MANDMIN + GOVDEPART + CIRCDIST + NEWRACE, data = lmdata, model = TRUE, fitted = TRUE, y = TRUE)
save(M11, file = "M11.Rdata")
remove(M11)
gc()

# personal demographics + year + plea + grid + mandmin + govt departures + district:race
M12 <- speedlm(SENTTOT ~ AGE + DISTRACE + MONSEX + NEWEDUC + SENTYR + NEWCNVTN + GRID + MANDMIN + GOVDEPART - 1, data = lmdata, model = TRUE, fitted = TRUE, y = TRUE)
save(M12, file = "M12.Rdata")
remove(M12)
gc()

# Now analyze within each district
dodistrict <-function(thisdist) {
  tmp <- lmdata %>%
    filter(CIRCDIST == thisdist) %>%
    droplevels
  MM <- lm(SENTTOT ~ AGE + MONSEX + NEWEDUC + SENTYR + NEWCNVTN + GRID + MANDMIN + GOVDEPART + NEWRACE, data = tmp)
  summaryMM <- summary(MM)
  res1 <- MM %>%
    coeftest(vcov. = vcovHC(MM, type = "HC1"))
  result <- res1 %>%
    tidy %>%
    mutate(district = thisdist) %>%
    dplyr::select("district", "term", "estimate", "std.error", "p.value") %>%
    mutate(r2 = summaryMM$adj.r.squared)
  confs <- confint(res1)
  result <- result %>%
    cbind(confs) %>%
    filter(grepl("NEWRACE",term))
  names(result) <- c("district","race","estimate","SE","p","adjr2","lower","upper")
  result <- result %>%
    mutate(race = str_replace_all(race,"NEWRACE",""))
  return(result)
}
MM <- bind_rows(pbmclapply(levels(lmdata$CIRCDIST), dodistrict, mc.cores = 35))
save(MM, file = "MM.Rdata")
remove(MM)
gc()

# Clean up
remove(list=c("dodistrict","lmdata"))